@@ -38,6 +38,8 @@
 #include <klocalizedstring.h>
 #include <Plasma/Package>
 #include <Plasma/PluginLoader>
+#include <PlasmaQuick/Dialog>
+
 #include <kactivities/controller.h>
 #include <kactivities/consumer.h>
 #include <ksycoca.h>
@@ -316,9 +318,7 @@ void ShellCorona::setShell(const QString &shell)
 
     connect(m_activityController, &KActivities::Controller::serviceStatusChanged, this, &ShellCorona::load, Qt::UniqueConnection);
 
-    if (m_activityController->serviceStatus() == KActivities::Controller::Running) {
-        load();
-    }
+    load();
 }
 
 
@@ -640,7 +640,8 @@ QString ShellCorona::shell() const
 void ShellCorona::load()
 {
     if (m_shell.isEmpty() ||
-        m_activityController->serviceStatus() != KActivities::Controller::Running) {
+        (m_activityController->serviceStatus() != KActivities::Controller::Running &&
+        !qApp->property("org.kde.KActivities.core.disableAutostart").toBool())) {
         return;
     }
 
@@ -821,42 +822,26 @@ void ShellCorona::showAlternativesForApplet(Plasma::Applet *applet)
     AlternativesHelper *helper = new AlternativesHelper(applet, qmlObj);
     qmlObj->rootContext()->setContextProperty(QStringLiteral("alternativesHelper"), helper);
 
-    m_alternativesObjects << qmlObj;
     qmlObj->completeInitialization();
-    connect(qmlObj->rootObject(), SIGNAL(visibleChanged(bool)),
-            this, SLOT(alternativesVisibilityChanged(bool)));
 
-    connect(applet, &Plasma::Applet::destroyedChanged, this, [this, qmlObj] (bool destroyed) {
+    auto dialog = qobject_cast<PlasmaQuick::Dialog*>(qmlObj->rootObject());
+    if (!dialog) {
+        qWarning() << "Alternatives UI does not inherit from Dialog";
+        delete qmlObj;
+        return;
+    }
+    connect(applet, &Plasma::Applet::destroyedChanged, qmlObj, [qmlObj] (bool destroyed) {
         if (!destroyed) {
             return;
         }
-        QMutableListIterator<KDeclarative::QmlObject *> it(m_alternativesObjects);
-        while (it.hasNext()) {
-            KDeclarative::QmlObject *obj = it.next();
-            if (obj == qmlObj) {
-                it.remove();
-                obj->deleteLater();
-            }
-        }
+        qmlObj->deleteLater();
     });
-}
-
-void ShellCorona::alternativesVisibilityChanged(bool visible)
-{
-    if (visible) {
-        return;
-    }
-
-    QObject *root = sender();
-
-    QMutableListIterator<KDeclarative::QmlObject *> it(m_alternativesObjects);
-    while (it.hasNext()) {
-        KDeclarative::QmlObject *obj = it.next();
-        if (obj->rootObject() == root) {
-            it.remove();
-            obj->deleteLater();
+    connect(dialog, &PlasmaQuick::Dialog::visibleChanged, qmlObj, [qmlObj](bool visible) {
+        if (visible) {
+            return;
         }
-    }
+        qmlObj->deleteLater();
+    });
 }
 
 void ShellCorona::unload()
@@ -1326,13 +1311,8 @@ void ShellCorona::handleContainmentAdded(Plasma::Containment *c)
 {
     connect(c, &Plasma::Containment::showAddWidgetsInterface,
             this, &ShellCorona::toggleWidgetExplorer);
-    // Why queued? this is usually triggered after a context menu closes
-    // due to its sync,modal nature it may eat some mouse event from the scene
-    // waiting a bit to create a new window, the dialog seems to reliably
-    // avoid the eating of one click in the panel after the context menu is gone
     connect(c, &Plasma::Containment::appletAlternativesRequested,
-            this, &ShellCorona::showAlternativesForApplet, Qt::QueuedConnection);
-
+            this, &ShellCorona::showAlternativesForApplet);
     connect(c, &Plasma::Containment::appletCreated, this, [this, c] (Plasma::Applet *applet) {
         executeSetupPlasmoidScript(c, applet);
     });
@@ -2039,6 +2019,7 @@ void ShellCorona::setupWaylandIntegration()
         }
     );
     registry->setup();
+    connection->roundtrip();
 }
 
 KWayland::Client::PlasmaShell *ShellCorona::waylandPlasmaShellInterface() const
